//*****************************************
// AT mega168 Para LCD Check Program
// CPU Clock:1MHz
//   iq[Yf[^@WāAIV[^(8MHz)ɂj
//
// File Name:lcd.c
//
//  2014/04/11 Taishi
//*****************************************

#include	<avr/pgmspace.h>
#include	<avr/io.h>

#include	"timer.h"
#include	"lcd.h"

// **********************************************************
// ************************** LCD ***************************
// **********************************************************
//
//******************************
//****** CG RAM DATA 5x7 *******
//******************************
static
unsigned char	lcd_cg_data[64] PROGMEM  = {
										//   12345
										0b00000000, //1				// Code:0x00(Not Use)
										0b00000000, //2
										0b00000000, //3
										0b00000000, //4
										0b00000000, //5
										0b00000000, //6
										0b00000000, //7
										0b00000000,

										0b00000000,				// Code:0x01
										0b00000000,				// Battery Left Side
										0b00000000,
										0b00000000,
										0b00000000,
										0b00000000,
										0b11111111,
										0b00000000,

										0b00000000,				// Code:0x02
										0b00000000,				// Battery Right Side
										0b00000000,
										0b00000000,
										0b00000000,
										0b11111111,
										0b11111111,
										0b00000000,

										0b00000000,				// Code:0x03
										0b00000000,				// LED
										0b00000000,
										0b00000000,
										0b11111111,
										0b11111111,
										0b11111111,
										0b00000000,

										0b00000000,				// Code:0x04
										0b00000000,
										0b00000000,
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b00000000,

										0b00000000,				// Code:0x05
										0b00000000,
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b00000000,

										0b00000000,				// Code:0x06
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b00000000,

										0b11111111,				// Code:0x07
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b11111111,
										0b00000000,

									};


//******************************
// LCD Data(4bit) Output
//  uc : Output Data(D4 -- D7)
//  rs : RS bit Data(0=Command, 1=Data)

// PD7:RS
// PD6:R/W*
// PD5:E
// PC3:DB7
// PC2:DB6
// PC1:DB5
// PC0:DB4
// *****************************
static
void	lcd_out_4bit(unsigned char uc, int rs)
{
	if((rs & 0x01) == 0){			// RS:0(Command)
		PORTD &= ~(1 << PD7);
	}else{							// RS:1(Data)
		PORTD |=  (1 << PD7);
	}

	PORTD |=  (1 << PD5);			// E:1
	PORTD &=  ~(1 << PD6);			// R/W*:0

	PORTC = ( (PORTC & ~((1 << PC0) | (1 << PC1) | (1 << PC2) | (1 << PC3))) | (uc >> 4) );	// Data


	PORTD &= ~(1 << PD5);			// E:0
}
//******************************
// LCD Command Output
//  uc : Command Data(D0 -- D7)
// *****************************
static
void	lcd_cmd_out(unsigned char uc)
{
	lcd_out_4bit(uc, 0);
	lcd_out_4bit((uc << 4), 0);
}
//******************************
// LCD Data Output
//  uc : Data(D0 -- D7)
// *****************************
static
void	lcd_data_out(unsigned char uc)
{
	lcd_out_4bit(uc, 1);
	lcd_out_4bit((uc << 4), 1);
}

//******************************
// LCD CG Data Set
// *****************************
static
void	lcd_cg_data_set(void)
{
	unsigned char	uc;
	unsigned char	i;

	lcd_cmd_out(0x40);							// CG RAM Address Set
	delay_16us(DELAY_LCD_us / 16);

	for(i=0;i<64;i++){
		uc = pgm_read_byte(lcd_cg_data + i);
		lcd_data_out(uc);
		delay_16us(DELAY_LCD_us / 16);
	}
}
//******************************
// Init. LCD
//
// PD7:RS
// PD6:R/W*
// PD5:E
// PC3:DB7
// PC2:DB6
// PC1:DB5
// PC0:DB4
// *****************************
void	init_lcd(void)
{
	DDRC = ((1 << DDC0) | (1 << DDC1) | (1 << DDC2) | (1 << DDC3));		// PC0--3:Output
	DDRD = ((1 << DDD5) | (1 << DDD6) | (1 << DDD7));					// PD5--7:Output

	delay_ms(15);					// Wait 15mS
	lcd_out_4bit(0x30,0);
	delay_ms(5);					// Wait 5mS
	lcd_out_4bit(0x30,0);
	delay_ms(1);					// Wait 1mS
	lcd_out_4bit(0x30,0);

	delay_ms(1);					// Wait 1mS
	lcd_out_4bit(0x20,0);			// 4bit Mode

	delay_ms(1);					// Wait 1mS
	lcd_cmd_out(0x28);				// 4bit Mode 2line 5x7dot
	delay_16us(DELAY_LCD_us / 16);
	lcd_cmd_out(0x08);				// Display OFF, Cursor OFF, Blink OFF
	delay_16us(DELAY_LCD_us / 16);
	lcd_cmd_out(0x01);				// Display Clear
	delay_ms(2);
	lcd_cmd_out(0x06);				// Inclement, Corsor Move
	delay_16us(DELAY_LCD_us / 16);
	lcd_cmd_out(0x02);				// Cursor Home
	delay_ms(2);
	lcd_cmd_out(0x0C);				// Display ON, Cursor OFF, Blink OFF
	delay_16us(DELAY_LCD_us / 16);

	lcd_cg_data_set();				// LCD CG Data Set
}
//******************************
// Clear Display
// *****************************
void	disp_clear(void)
{
	lcd_cmd_out(0x01);				// Display Clear
	delay_ms(2);
	lcd_cmd_out(0x06);				// Inclement, Corsor Move
	delay_16us(DELAY_LCD_us / 16);
	lcd_cmd_out(0x02);				// Cursor Home
	delay_ms(2);
	lcd_cmd_out(0x0C);				// Display ON, Cursor OFF, Blink OFF
	delay_16us(DELAY_LCD_us / 16);

}

//******************************
// display LCD(16char x 2Line)
//  pos_x : Position X(0 -- 15)
//  pos_y : Position Y(0 -- 1)
//  ucp : String(termnate 0x00)
// *****************************
void	disp_lcd(unsigned char pos_x, unsigned char pos_y, unsigned char *ucp)
{
	unsigned char	pos;

	if((pos_x <= 15)&&(pos_y <= 1)){
		pos = pos_x;
		if(pos_y == 1){						// 2'nd Line
			pos += 0x40;
		}
		lcd_cmd_out(pos | 0x80);			// Set DDRAM ADR.
		delay_16us(DELAY_LCD_us / 16);

		while(*ucp != 0x00){				// String Data Out
			lcd_data_out(*ucp);
			ucp++;
			delay_16us(DELAY_LCD_us / 16);
		}
	}
}
